@@ -36,7 +36,7 @@ static std::unique_ptr<PegasusPPBA> ppba(new PegasusPPBA());
 
 PegasusPPBA::PegasusPPBA() : FI(this), WI(this)
 {
-    setVersion(1, 2);
+    setVersion(1, 3);
     lastSensorData.reserve(PA_N);
     lastConsumptionData.reserve(PS_N);
     lastMetricsData.reserve(PC_N);
@@ -67,7 +67,7 @@ bool PegasusPPBA::initProperties()
     QuadOutSP[INDI_ENABLED].fill("QUADOUT_ON", "Enable", ISS_OFF);
     QuadOutSP[INDI_DISABLED].fill("QUADOUT_OFF", "Disable", ISS_OFF);
     QuadOutSP.fill(getDeviceName(), "QUADOUT_POWER", "Quad Output", MAIN_CONTROL_TAB,
-                       IP_RW, ISR_ATMOST1, 60, IPS_IDLE);
+                   IP_RW, ISR_ATMOST1, 60, IPS_IDLE);
 
     // Adjustable Power
     //    IUFillSwitch(&AdjOutS[INDI_ENABLED], "ADJOUT_ON", "Enable", ISS_OFF);
@@ -83,12 +83,12 @@ bool PegasusPPBA::initProperties()
     AdjOutVoltSP[ADJOUT_9V].fill("ADJOUT_9V", "9V", ISS_OFF);
     AdjOutVoltSP[ADJOUT_12V].fill("ADJOUT_12V", "12V", ISS_OFF);
     AdjOutVoltSP.fill(getDeviceName(), "ADJOUT_VOLTAGE", "Adj voltage", MAIN_CONTROL_TAB, IP_RW,
-                       ISR_1OFMANY, 60, IPS_IDLE);
+                      ISR_1OFMANY, 60, IPS_IDLE);
 
     // Reboot
     RebootSP[0].fill("REBOOT", "Reboot Device", ISS_OFF);
     RebootSP.fill(getDeviceName(), "REBOOT_DEVICE", "Device", MAIN_CONTROL_TAB, IP_RW, ISR_ATMOST1,
-                       60, IPS_IDLE);
+                  60, IPS_IDLE);
 
     // Power Sensors
     PowerSensorsNP[SENSOR_VOLTAGE].fill("SENSOR_VOLTAGE", "Voltage (V)", "%4.2f", 0, 999, 100, 0);
@@ -101,7 +101,7 @@ bool PegasusPPBA::initProperties()
     PowerSensorsNP[SENSOR_DEWA_CURRENT].fill("SENSOR_DEWA_CURRENT", "DewA current (A)", "%4.2f", 0, 999, 100, 0);
     PowerSensorsNP[SENSOR_DEWB_CURRENT].fill("SENSOR_DEWB_CURRENT", "DewB current (A)", "%4.2f", 0, 999, 100, 0);
     PowerSensorsNP.fill(getDeviceName(), "POWER_SENSORS", "Sensors", MAIN_CONTROL_TAB, IP_RO,
-                       60, IPS_IDLE);
+                        60, IPS_IDLE);
 
     PowerWarnLP[0].fill("POWER_WARN_ON", "Current Overload", IPS_IDLE);
     PowerWarnLP.fill(getDeviceName(), "POWER_WARM", "Power Warn", MAIN_CONTROL_TAB, IPS_IDLE);
@@ -110,8 +110,8 @@ bool PegasusPPBA::initProperties()
     LedIndicatorSP[INDI_ENABLED].fill("LED_ON", "Enable", ISS_ON);
     LedIndicatorSP[INDI_DISABLED].fill("LED_OFF", "Disable", ISS_OFF);
     LedIndicatorSP.fill(getDeviceName(), "LED_INDICATOR", "LED Indicator", MAIN_CONTROL_TAB,
-                       IP_RW,
-                       ISR_1OFMANY, 60, IPS_IDLE);
+                        IP_RW,
+                        ISR_1OFMANY, 60, IPS_IDLE);
 
     ////////////////////////////////////////////////////////////////////////////
     /// Power Group
@@ -133,7 +133,7 @@ bool PegasusPPBA::initProperties()
     AutoDewSP[INDI_ENABLED].fill("INDI_ENABLED", "Enabled", ISS_OFF);
     AutoDewSP[INDI_DISABLED].fill("INDI_DISABLED", "Disabled", ISS_OFF);
     AutoDewSP.fill(getDeviceName(), "AUTO_DEW", "Auto Dew", DEW_TAB, IP_RW, ISR_1OFMANY, 60,
-                       IPS_IDLE);
+                   IPS_IDLE);
 
     AutoDewSettingsNP[AUTO_DEW_AGGRESSION].fill("AGGRESSION", "Aggresiveness (%)", "%.2f", 0, 100, 10, 0);
     AutoDewSettingsNP.fill(getDeviceName(), "AUTO_DEW_SETTINGS", "Auto Dew Settings", DEW_TAB, IP_RW, 60, IPS_IDLE);
@@ -150,7 +150,7 @@ bool PegasusPPBA::initProperties()
     FirmwareTP[FIRMWARE_VERSION].fill("VERSION", "Version", "NA");
     FirmwareTP[FIRMWARE_UPTIME].fill("UPTIME", "Uptime (h)", "NA");
     FirmwareTP.fill(getDeviceName(), "FIRMWARE_INFO", "Firmware", FIRMWARE_TAB, IP_RO, 60,
-                     IPS_IDLE);
+                    IPS_IDLE);
 
     ////////////////////////////////////////////////////////////////////////////
     /// Environment Group
@@ -167,7 +167,7 @@ bool PegasusPPBA::initProperties()
     // Max Speed
     FocuserSettingsNP[SETTING_MAX_SPEED].fill("SETTING_MAX_SPEED", "Max Speed (%)", "%.f", 0, 900, 100, 400);
     FocuserSettingsNP.fill(getDeviceName(), "FOCUSER_SETTINGS", "Settings", FOCUS_TAB,
-                       IP_RW, 60, IPS_IDLE);
+                           IP_RW, 60, IPS_IDLE);
 
     // Stepping
     FocuserDriveSP[STEP_FULL].fill("STEP_FULL", "Full", ISS_OFF);
@@ -175,7 +175,7 @@ bool PegasusPPBA::initProperties()
     FocuserDriveSP[STEP_FORTH].fill("STEP_FORTH", "1/4", ISS_OFF);
     FocuserDriveSP[STEP_EIGHTH].fill("STEP_EIGHTH", "1/8", ISS_OFF);
     FocuserDriveSP.fill(getDeviceName(), "FOCUSER_DRIVE", "Microstepping", FOCUS_TAB,
-                       IP_RW, ISR_1OFMANY, 60, IPS_IDLE);
+                        IP_RW, ISR_1OFMANY, 60, IPS_IDLE);
 
     ////////////////////////////////////////////////////////////////////////////
     /// Serial Connection
@@ -398,7 +398,10 @@ bool PegasusPPBA::ISNewSwitch(const char * dev, const char * name, ISState * sta
             char cmd[PEGASUS_LEN] = {0}, res[PEGASUS_LEN] = {0};
             snprintf(cmd, PEGASUS_LEN, "P2:%d", adjv);
             if (sendCommand(cmd, res))
+            {
                 AdjOutVoltSP.setState(IPS_OK);
+                saveConfig(AdjOutVoltSP);
+            }
             else
             {
                 AdjOutVoltSP.reset();
@@ -517,7 +520,8 @@ bool PegasusPPBA::ISNewNumber(const char * dev, const char * name, double values
         // Auto Dew Settings
         if (AutoDewSettingsNP.isNameMatch(name))
         {
-            AutoDewSettingsNP.setState(setAutoDewAggression(static_cast<uint8_t>(values[AUTO_DEW_AGGRESSION] / 100.0 * 255.0)) ? IPS_OK : IPS_ALERT);
+            AutoDewSettingsNP.setState(setAutoDewAggression(static_cast<uint8_t>(values[AUTO_DEW_AGGRESSION] / 100.0 * 255.0)) ?
+                                       IPS_OK : IPS_ALERT);
             if (AutoDewSettingsNP.getState() == IPS_OK)
                 AutoDewSettingsNP.update(values, names, n);
             AutoDewSettingsNP.apply();
@@ -663,6 +667,7 @@ bool PegasusPPBA::saveConfigItems(FILE * fp)
     WI::saveConfigItems(fp);
     AutoDewSP.save(fp);
     AutoDewSettingsNP.save(fp);
+    AdjOutVoltSP.save(fp);
     return true;
 }
 
@@ -717,14 +722,16 @@ bool PegasusPPBA::getSensorData()
         PowerSensorsNP[SENSOR_VOLTAGE].setValue(std::stod(result[PA_VOLTAGE]));
         PowerSensorsNP[SENSOR_CURRENT].setValue(std::stod(result[PA_CURRENT]) / 65.0);
         PowerSensorsNP.setState(IPS_OK);
-        if (lastSensorData[PA_VOLTAGE] != result[PA_VOLTAGE] || lastSensorData[PA_CURRENT] != result[PA_CURRENT])
+        if (lastSensorData.size() < PA_N ||
+                lastSensorData[PA_VOLTAGE] != result[PA_VOLTAGE] || lastSensorData[PA_CURRENT] != result[PA_CURRENT])
             PowerSensorsNP.apply();
 
         // Environment Sensors
         setParameterValue("WEATHER_TEMPERATURE", std::stod(result[PA_TEMPERATURE]));
         setParameterValue("WEATHER_HUMIDITY", std::stod(result[PA_HUMIDITY]));
         setParameterValue("WEATHER_DEWPOINT", std::stod(result[PA_DEW_POINT]));
-        if (lastSensorData[PA_TEMPERATURE] != result[PA_TEMPERATURE] ||
+        if (lastSensorData.size() < PA_N ||
+                lastSensorData[PA_TEMPERATURE] != result[PA_TEMPERATURE] ||
                 lastSensorData[PA_HUMIDITY] != result[PA_HUMIDITY] ||
                 lastSensorData[PA_DEW_POINT] != result[PA_DEW_POINT])
         {
@@ -738,7 +745,9 @@ bool PegasusPPBA::getSensorData()
         QuadOutSP[INDI_ENABLED].setState((std::stoi(result[PA_PORT_STATUS]) == 1) ? ISS_ON : ISS_OFF);
         QuadOutSP[INDI_DISABLED].setState((std::stoi(result[PA_PORT_STATUS]) == 1) ? ISS_OFF : ISS_ON);
         QuadOutSP.setState((std::stoi(result[6]) == 1) ? IPS_OK : IPS_IDLE);
-        if (lastSensorData[PA_PORT_STATUS] != result[PA_PORT_STATUS])
+        if (lastSensorData.size() < PA_N ||
+                lastSensorData[PA_PORT_STATUS] != result[PA_PORT_STATUS] ||
+                lastSensorData[6] != result[6]) // Check index 6 as well, assuming it's covered by PA_N
             QuadOutSP.apply();
 
         // Adjustable Power Status
@@ -760,26 +769,30 @@ bool PegasusPPBA::getSensorData()
             AdjOutVoltSP[ADJOUT_9V].setState((std::stoi(result[PA_PWRADJ]) == 9) ? ISS_ON : ISS_OFF);
             AdjOutVoltSP[ADJOUT_12V].setState((std::stoi(result[PA_PWRADJ]) == 12) ? ISS_ON : ISS_OFF);
         }
-        if (lastSensorData[PA_PWRADJ] != result[PA_PWRADJ] || lastSensorData[PA_ADJ_STATUS] != result[PA_ADJ_STATUS])
+        if (lastSensorData.size() < PA_N ||
+                lastSensorData[PA_PWRADJ] != result[PA_PWRADJ] || lastSensorData[PA_ADJ_STATUS] != result[PA_ADJ_STATUS])
             AdjOutVoltSP.apply();
 
         // Power Warn
         PowerWarnLP[0].setState((std::stoi(result[PA_PWR_WARN]) == 1) ? IPS_ALERT : IPS_OK);
         PowerWarnLP.setState((std::stoi(result[PA_PWR_WARN]) == 1) ? IPS_ALERT : IPS_OK);
-        if (lastSensorData[PA_PWR_WARN] != result[PA_PWR_WARN])
+        if (lastSensorData.size() < PA_N ||
+                lastSensorData[PA_PWR_WARN] != result[PA_PWR_WARN])
             PowerWarnLP.apply();
 
         // Dew PWM
         DewPWMNP[DEW_PWM_A].setValue(std::stod(result[PA_DEW_1]) / 255.0 * 100.0);
         DewPWMNP[DEW_PWM_B].setValue(std::stod(result[PA_DEW_2]) / 255.0 * 100.0);
-        if (lastSensorData[PA_DEW_1] != result[PA_DEW_1] || lastSensorData[PA_DEW_2] != result[PA_DEW_2])
+        if (lastSensorData.size() < PA_N ||
+                lastSensorData[PA_DEW_1] != result[PA_DEW_1] || lastSensorData[PA_DEW_2] != result[PA_DEW_2])
             DewPWMNP.apply();
 
         // Auto Dew
         AutoDewSP[INDI_DISABLED].setState((std::stoi(result[PA_AUTO_DEW]) == 1) ? ISS_OFF : ISS_ON);
         AutoDewSP[INDI_ENABLED].setState((std::stoi(result[PA_AUTO_DEW]) == 1) ? ISS_ON : ISS_OFF);
         AutoDewSP.setState((std::stoi(result[PA_AUTO_DEW]) == 1) ? IPS_OK : IPS_IDLE);
-        if (lastSensorData[PA_AUTO_DEW] != result[PA_AUTO_DEW])
+        if (lastSensorData.size() < PA_N ||
+                lastSensorData[PA_AUTO_DEW] != result[PA_AUTO_DEW])
             AutoDewSP.apply();
 
         lastSensorData = result;
@@ -810,8 +823,10 @@ bool PegasusPPBA::getConsumptionData()
         PowerSensorsNP[SENSOR_AMP_HOURS].setValue(std::stod(result[PS_AMP_HOURS]));
         PowerSensorsNP[SENSOR_WATT_HOURS].setValue(std::stod(result[PS_WATT_HOURS]));
         PowerSensorsNP.setState(IPS_OK);
-        if (lastConsumptionData[PS_AVG_AMPS] != result[PS_AVG_AMPS] || lastConsumptionData[PS_AMP_HOURS] != result[PS_AMP_HOURS]
-                || lastConsumptionData[PS_WATT_HOURS] != result[PS_WATT_HOURS])
+        if (lastConsumptionData.size() < PS_N ||
+                lastConsumptionData[PS_AVG_AMPS] != result[PS_AVG_AMPS] ||
+                lastConsumptionData[PS_AMP_HOURS] != result[PS_AMP_HOURS] ||
+                lastConsumptionData[PS_WATT_HOURS] != result[PS_WATT_HOURS])
             PowerSensorsNP.apply();
 
         lastConsumptionData = result;
@@ -827,14 +842,14 @@ bool PegasusPPBA::getAutoDewAggression()
     char res[PEGASUS_LEN] = {0};
     if (sendCommand("DA", res))
     {
-        
+
         uint32_t value = 0;
         sscanf(res, "%*[^:]:%d", &value);
         AutoDewSettingsNP[AUTO_DEW_AGGRESSION].setValue(100 * value / 255);
     }
     else
         AutoDewSettingsNP.setState(IPS_ALERT);
-        
+
     AutoDewSettingsNP.apply();
     return AutoDewSettingsNP.getState() != IPS_ALERT;
 }
@@ -860,14 +875,15 @@ bool PegasusPPBA::getMetricsData()
         PowerSensorsNP[SENSOR_DEWA_CURRENT].setValue(std::stod(result[PC_DEWA_CURRENT]));
         PowerSensorsNP[SENSOR_DEWB_CURRENT].setValue(std::stod(result[PC_DEWB_CURRENT]));
         PowerSensorsNP.setState(IPS_OK);
-        if (lastMetricsData[PC_TOTAL_CURRENT] != result[PC_TOTAL_CURRENT] ||
+        if (lastMetricsData.size() < PC_N ||
+                lastMetricsData[PC_TOTAL_CURRENT] != result[PC_TOTAL_CURRENT] ||
                 lastMetricsData[PC_12V_CURRENT] != result[PC_12V_CURRENT] ||
                 lastMetricsData[PC_DEWA_CURRENT] != result[PC_DEWA_CURRENT] ||
                 lastMetricsData[PC_DEWB_CURRENT] != result[PC_DEWB_CURRENT])
             PowerSensorsNP.apply();
 
         std::chrono::milliseconds uptime(std::stol(result[PC_UPTIME]));
-        using dhours = std::chrono::duration<double, std::ratio<3600>>;
+        using dhours = std::chrono::duration<double, std::ratio<3600 >>;
         std::stringstream ss;
         ss << std::fixed << std::setprecision(3) << dhours(uptime).count();
         FirmwareTP[FIRMWARE_UPTIME].setText(ss.str().c_str());
@@ -901,7 +917,7 @@ IPState PegasusPPBA::MoveAbsFocuser(uint32_t targetTicks)
 //////////////////////////////////////////////////////////////////////
 IPState PegasusPPBA::MoveRelFocuser(FocusDirection dir, uint32_t ticks)
 {
-    return MoveAbsFocuser(dir == FOCUS_INWARD ? FocusAbsPosN[0].value - ticks : FocusAbsPosN[0].value + ticks);
+    return MoveAbsFocuser(dir == FOCUS_INWARD ? FocusAbsPosNP[0].getValue() - ticks : FocusAbsPosNP[0].getValue() + ticks);
 }
 
 //////////////////////////////////////////////////////////////////////
@@ -981,7 +997,7 @@ bool PegasusPPBA::getXMCStartupData()
     {
         uint32_t position = 0;
         sscanf(res, "%*[^#]#%d", &position);
-        FocusAbsPosN[0].value = position;
+        FocusAbsPosNP[0].setValue(position);
     }
 
     // Max speed
@@ -1011,18 +1027,18 @@ void PegasusPPBA::queryXMC()
     if (sendCommand("XS:2", res))
         sscanf(res, "%*[^#]#%d", &position);
 
-    uint32_t lastPosition = FocusAbsPosN[0].value;
-    FocusAbsPosN[0].value = position;
+    uint32_t lastPosition = FocusAbsPosNP[0].getValue();
+    FocusAbsPosNP[0].setValue(position);
 
-    if (FocusAbsPosNP.s == IPS_BUSY && motorRunning == 0)
+    if (FocusAbsPosNP.getState() == IPS_BUSY && motorRunning == 0)
     {
-        FocusAbsPosNP.s = IPS_OK;
-        FocusRelPosNP.s = IPS_OK;
-        IDSetNumber(&FocusAbsPosNP, nullptr);
-        IDSetNumber(&FocusRelPosNP, nullptr);
+        FocusAbsPosNP.setState(IPS_OK);
+        FocusRelPosNP.setState(IPS_OK);
+        FocusAbsPosNP.apply();
+        FocusRelPosNP.apply();
     }
     else if (lastPosition != position)
-        IDSetNumber(&FocusAbsPosNP, nullptr);
+        FocusAbsPosNP.apply();
 
 }
 //////////////////////////////////////////////////////////////////////
@@ -1037,4 +1053,3 @@ std::vector<std::string> PegasusPPBA::split(const std::string &input, const std:
           last;
     return {first, last};
 }
-
